@@ -31,13 +31,17 @@ def is_private_comment(comment):
     else:
         return False
 
+
 def comment2lines(node):
+    comments = []
     for line in node.text.splitlines():
         if re.match('^\s*/?\*{1,2} ?', line):  # starts with '/**' or '/*' or '*' ?
             line = re.sub('\s*\*/.*$', '', line)  # remove '*/' of tail
             line = re.sub('^\s*/?\*{1,2} ?', '', line)  # remove '/**' or '/*' or '*' of top
 
-            yield line
+            comments.append(line)
+
+    return "\n".join(comments).strip().splitlines()
 
 
 def to_s(node):
@@ -127,10 +131,11 @@ class PHPDocWriter(Directive):
         self.add_line(u'.. %s:%s:: %s' % (domain, directive, name), indent_level)
         self.add_line('')
 
-    def add_directive(self, directive, name, comment_node, indent_level=0):
+    def add_directive(self, directive, name, comment_node, indent_level=0, force=False):
         if is_private_comment(comment_node):
-            pass  # skipped
-        elif is_comment(comment_node) or 'undoc-members' in self.options:
+            if force is True:
+                self.add_directive_header(directive, name, indent_level)
+        elif is_comment(comment_node) or 'undoc-members' in self.options or force is True:
             self.add_directive_header(directive, name, indent_level)
 
             if is_comment(comment_node):
@@ -158,15 +163,20 @@ class PHPAutodocDirectiveBase(PHPDocWriter, AutodocCache):
         srcdir = self.state.document.settings.env.srcdir
         filename = os.path.join(srcdir, self.options['filename'])
         if not os.path.exists(filename):
-            msg = '%s cannot read source code:' % (self.directive_name, self.options['filename'])
+            msg = '%s cannot read source code: %s' % (self.directive_name, self.options['filename'])
             return [self.state.document.reporter.warning(msg, line=self.lineno)]
 
         tree = self.parse_code(filename)
         self.traverse(tree)
         self.state.document.settings.env.note_dependency(filename)
 
-        for line in self.content:
-            self.add_line(line, 1)
+        if self.content:
+            for line in self.content:
+                self.add_line(line, 1)
+
+            self.add_line('')
+
+        self.add_line('')
 
         node = nodes.paragraph()
         node.document = self.state.document
@@ -188,13 +198,27 @@ class PHPAutodocDirectiveBase(PHPDocWriter, AutodocCache):
                 else:
                     self.add_directive('class', node.name, last_node, indent)
 
+                    if 'members' in self.options:
+                        self.traverse_all(node.nodes, indent + 1)
+            elif isinstance(node, ast.Interface):
+                if is_private_comment(last_node):
+                    pass
+                else:
+                    self.add_directive('interface', node.name, last_node, indent)
+
                     if 'members' in self.options:
                         self.traverse_all(node.nodes, indent + 1)
             elif isinstance(node, ast.Method):
-                self.add_directive('method', to_s(node), last_node, indent)
+                if 'protected' in node.modifiers or 'private' in node.modifiers:
+                    pass
+                else:
+                    self.add_directive('method', to_s(node), last_node, indent)
             elif isinstance(node, ast.ClassVariables):
-                for variable in node.nodes:
-                    self.add_directive('attr', variable.name, last_node, indent)
+                if 'protected' in node.modifiers or 'private' in node.modifiers:
+                    pass
+                else:
+                    for variable in node.nodes:
+                        self.add_directive('attr', variable.name, last_node, indent)
 
             last_node = node
 
@@ -219,12 +243,20 @@ class PHPAutoClassDirective(PHPAutodocDirectiveBase):
         last_node = None
         for node in tree:
             if isinstance(node, ast.Class) and node.name in self.targets:
-                self.traverse_all([last_node, node])
+                self.add_directive('class', node.name, last_node, indent, force=True)
+
+                if 'members' in self.options:
+                    self.traverse_all(node.nodes, indent + 1)
+            elif isinstance(node, ast.Interface) and node.name in self.targets:
+                self.add_directive('interface', node.name, last_node, indent, force=True)
+
+                if 'members' in self.options:
+                    self.traverse_all(node.nodes, indent + 1)
 
             last_node = node
 
 
-def setup(app): 
+def setup(app):
     classes = [PHPAutoModuleDirective, PHPAutoClassDirective]
     for cls in classes:
         app.add_directive(cls.directive_name, cls)
